package com.galaxis.manatee.controller;

import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunCostCollectionDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunEquipmentDO;
import com.galaxis.manatee.service.ChuanyunService;
import com.galaxis.manatee.task.ChuanyunBasicDataScheduledTask;
import com.galaxis.manatee.task.ChuanyunHumanResourceScheduledTask;
import com.galaxis.manatee.util.ExcelUtils;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSON;
import net.sf.json.JSONObject;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.Async;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 提供手工触发更新氚云数据的接口
 * @author zcj
 * @version 0.1
 * @date 2020/6/7 10:26 上午
 */
@RestController
@Slf4j
public class GetController {

    private final ChuanyunHumanResourceScheduledTask chuanyunHumanResourceScheduledTask;
    private final ChuanyunBasicDataScheduledTask chuanyunBasicDataScheduledTask;
    private final ChuanyunService chuanyunService;

    public GetController(ChuanyunHumanResourceScheduledTask chuanyunHumanResourceScheduledTask, ChuanyunBasicDataScheduledTask chuanyunBasicDataScheduledTask, ChuanyunService chuanyunService) {
        this.chuanyunHumanResourceScheduledTask = chuanyunHumanResourceScheduledTask;
        this.chuanyunBasicDataScheduledTask = chuanyunBasicDataScheduledTask;
        this.chuanyunService = chuanyunService;
    }

    /**
     * 手动更新chuanyunPayRoll
     */
    @Async
    @GetMapping("/test/getChuanyunPayroll")
    public void getChuanyunPayroll(){
        chuanyunHumanResourceScheduledTask.getPayrollManually();
    }

    /**
     * 手动更新日志
     */
    @Async
    @GetMapping("/test/getUpdateLog")
    public void getUpdateLog() {
        chuanyunHumanResourceScheduledTask.getUpdateLog();
    }

    /**
     * 手动更新项目组成员
     */
    @Async
    @GetMapping("/test/getProjectMember")
    public void getProjectMember() {
        chuanyunService.updateProjectMember();
    }

    /**
     * 手动更新人员-归属公司
     */
    @Async
    @GetMapping("/test/getUserCompany")
    public void getUserCompanyManually() {
        chuanyunBasicDataScheduledTask.getUserCompanyManually();
    }

    /**
     * 手动更新人员信息
     */
    @Async
    @GetMapping("/test/getUserList")
    public void getUserListManually() {
        chuanyunBasicDataScheduledTask.getUserListManually();
    }

    /**
     * 手动更新公司信息
     */
    @Async
    @GetMapping("/test/getCompany")
    public void getCompanyManually() {
        chuanyunBasicDataScheduledTask.getCompanyManually();
    }

    /**
     * 手动更新集团项目信息
     */
    @Async
    @GetMapping("/test/getGroupProject")
    public void getGroupProjectManually() {
        chuanyunBasicDataScheduledTask.getGroupProjectManually();
    }

    @Async
    @GetMapping("/test/getCostCollectionManually")
    public void getCostCollectionManually() {
        chuanyunBasicDataScheduledTask.getCostCollectionManually();
    }

    @Async
    @GetMapping("/test/getReimbursementItemsManually")
    public void getReimbursementItemsManually() {
        chuanyunBasicDataScheduledTask.getReimbursementItemsManually();
    }

    /**
     * 获取人员费用归集
     * @return
     */
    @GetMapping("/test/getAllCostCollection")
    public ResponseEntity getAllCostCollection() {
        List<ChuanyunCostCollectionDO> list = chuanyunService.getAllCostCollection();
        return ResponseEntity.ok(list);
    }

    /**
     * 更新人员所属部门
     * @param code
     */
    @GetMapping("/test/updateProjectDept")
    public void updateProjectDept(@RequestParam("code") String code) {
        chuanyunService.updateChuanyunDept(code);
    }

    /**
     * 导入设备配置信息
     * @param file
     */
    @PostMapping("/test/import")
    public void importEquipment(@RequestParam(value="uploadFile", required = false) MultipartFile file){
        List<ChuanyunEquipmentDO> list = ExcelUtils.readExcel("", ChuanyunEquipmentDO.class, file);
        chuanyunService.importEquipment(list);
    }

//    @GetMapping("/test/updateMember")
//    public void updateMember(){
//        chuanyunBasicDataScheduledTask.chuanyunUpdateProjectMemberManually();
//    }


}
