package com.galaxis.manatee.task;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.galaxis.capsule.dto.PageDTO;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.*;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.manager.ClawFeign;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeDeserializer;
import com.galaxis.manatee.util.ChuanyunLocalDateTimeSerializer;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class U9InfoTask {

    private final Integer pageSize=20;
    private final ChuanYunManager chuanYunManager;
    private final ClawFeign clawFeign;
    private final ChuanyunBudgetStatisticsDao chuanyunBudgetStatisticsDao;


    public U9InfoTask(ChuanYunManager chuanYunManager, ClawFeign clawFeign, ChuanyunBudgetStatisticsDao chuanyunBudgetStatisticsDao) {
        this.chuanYunManager = chuanYunManager;
        this.clawFeign = clawFeign;
        this.chuanyunBudgetStatisticsDao = chuanyunBudgetStatisticsDao;

    }

    @Scheduled(fixedDelay = 3600000L)
    private void getDataFromU9() {
        savePurCost();
        saveCashFlow();
        saveSupplier();
        saveLaborCost();
        saveBudgetStatistics();
    }

    /**
     * 更新项目物料成本
     */
    private void savePurCost() {
        var objectMapper=new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag=true;
        while (flag) {
            Map<String,String> map=new HashMap<>(1);
            map.put("page",String.valueOf(page));
            map.put("size",String.valueOf(size));
            //获取物料成本
            PageDTO<ChuanyunPurCostDTO> pageInfo = clawFeign.findChuanyunPurCost(map);
            if (page<=pageInfo.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunPurCostDTO> list = pageInfo.getContent();
            list.forEach(chuanyunPurCostDTO -> {
                try {
                    if(chuanyunPurCostDTO != null) {
                        PurCostDTO purCostDTO = new PurCostDTO(chuanyunPurCostDTO);
                        var purcostString=objectMapper.writeValueAsString(purCostDTO);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        //根据项目号判断是否存在
                        List<String> macthers = new ArrayList<>();
                        macthers.add("F0000001_2,"+chuanyunPurCostDTO.getProjectcode());
                        var filter= Filter.instance(0,1,true,"And",macthers);
                        var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunPurCostDO.SCHEMA_CODE,filter);
                        if (chuanyunFindAllResponse.getReturnData() == null) {
                            //新增
                            chuanyunSaveDTO =chuanYunManager.save(ChuanyunPurCostDO.SCHEMA_CODE,purcostString,true);
                        }else {
                            List<ChuanyunPurCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                            ChuanyunPurCostDO chuanyunPurCostDO = result.get(0);
                            //更新
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunPurCostDO.SCHEMA_CODE,chuanyunPurCostDO.getObjectId(),purcostString);
                        }
                        if(chuanyunSaveDTO.getSuccessful()){
//                            log.info("新增项目物料成本成功");
                        }else{
                            log.warn("新增项目物料成本失败");
                        }
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
    }

    /**
     * 更新项目收付款
     */
    private void saveCashFlow() {
        var objectMapper=new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag=true;
        while (flag) {
            Map<String,String> map=new HashMap<>(1);
            map.put("page",String.valueOf(page));
            map.put("size",String.valueOf(size));
            //获取收付款
            PageDTO<ChuanyunProjectCashFlowDTO> pageInfo = clawFeign.findChuanyunProjectCashFlow(map);
            if (page<=pageInfo.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunProjectCashFlowDTO> list = pageInfo.getContent();
            list.forEach(chuanyunProjectCashFlowDTO -> {
                try {
                    if(chuanyunProjectCashFlowDTO != null) {
                        CashFlowDTO cashFlowDTO = new CashFlowDTO(chuanyunProjectCashFlowDTO);
                        var purcostString=objectMapper.writeValueAsString(cashFlowDTO);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        //根据项目号判断是否存在
                        List<String> macthers = new ArrayList<>();
                        macthers.add("F0000001_2,"+chuanyunProjectCashFlowDTO.getProjectcode());
                        var filter= Filter.instance(0,1,true,"And",macthers);
                        var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunProjectCashFlowDO.SCHEMA_CODE,filter);
                        if (chuanyunFindAllResponse.getReturnData() == null) {
                            //新增
                            chuanyunSaveDTO =chuanYunManager.save(ChuanyunProjectCashFlowDO.SCHEMA_CODE,purcostString,true);
                        }else {
                            List<ChuanyunProjectCashFlowDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                            ChuanyunProjectCashFlowDO chuanyunProjectCashFlowDO = result.get(0);
                            //更新
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunProjectCashFlowDO.SCHEMA_CODE,chuanyunProjectCashFlowDO.getObjectId(),purcostString);
                        }
                        if(chuanyunSaveDTO.getSuccessful()){
//                            log.info("新增项目收付款成功");
                        }else{
                            log.warn("新增项目收付款失败");
                        }
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
    }

    /**
     * U9供应商信息
     */
    private void saveSupplier() {
        var objectMapper=new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag=true;
        while (flag) {
            Map<String,String> map=new HashMap<>(1);
            map.put("page",String.valueOf(page));
            map.put("size",String.valueOf(size));
            //获取供应商信息
            PageDTO<ChuanyunSupplierDTO> pageInfo=clawFeign.findChuanyunSupplier(map);
            if (page<=pageInfo.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunSupplierDTO> list = pageInfo.getContent();
            list.forEach(chuanyunSupplierDTO -> {
                SupplierDTO supplierDTO = new SupplierDTO(chuanyunSupplierDTO);
                try {
                    var supplierString = objectMapper.writeValueAsString(supplierDTO);
                    ChuanyunSaveDTO chuanyunSaveDTO;
                    //根据编码判断供应商是否存在
                    List<String> macthers = new ArrayList<>();
                    macthers.add("F0000001_2,"+chuanyunSupplierDTO.getName());
                    var filter= Filter.instance(0,1,true,"And",macthers);
                    var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunSupplierDO.SCHEMA_CODE,filter);
                    if (chuanyunFindAllResponse.getReturnData() == null) {
                        chuanyunSaveDTO=chuanYunManager.save(ChuanyunSupplierDO.SCHEMA_CODE,supplierString,true);
                    }else {
                        List<ChuanyunSupplierDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                        chuanyunSaveDTO=chuanYunManager.update(ChuanyunSupplierDO.SCHEMA_CODE,result.get(0).getObjectId(),supplierString);
                    }
                    if(chuanyunSaveDTO.getSuccessful()){
//                        log.info("新增供应商信息成功");
                    }else{
                        log.warn("新增供应商信息失败");
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }

            });
        }
    }

    /**
     * 新增人力成本
     */
    private void saveLaborCost() {
        var objectMapper=new ObjectMapper();
        var page = 1;
        var size = 20;
        var flag=true;
        while (flag) {
            Map<String,String> map=new HashMap<>(1);
            map.put("page",String.valueOf(page));
            map.put("size",String.valueOf(size));
            //获取物料成本
            PageDTO<ChuanyunLaborCostDTO> pageInfo = clawFeign.findChuanyunLaborCost(map);
            if (page<=pageInfo.getTotalPages()){
                page += 1;
            }else {
                flag = false;
                break;
            }
            List<ChuanyunLaborCostDTO> list = pageInfo.getContent();
            list.forEach(chuanyunLaborCostDTO -> {
                try {
                    if(chuanyunLaborCostDTO != null) {
                        DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm:ss");
                        String newDate = dateTimeFormatter.format(chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod());
                        LocalDateTime dt = LocalDateTime.parse(newDate,dateTimeFormatter);
                        chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().setPeriod(dt);
                        LaborCostDTO laborCostDTO = new LaborCostDTO(chuanyunLaborCostDTO);
                        laborCostDTO.setDataType("U9");
                        var purcostString=objectMapper.writeValueAsString(laborCostDTO);
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        //根据项目号判断是否存在
                        List<String> matchers = new ArrayList<>();
                        matchers.add("F0000005_2,"+chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getProjectcode());
                        matchers.add("F0000002_2,"+chuanyunLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod());
                        var filter= Filter.instance(0,1,true,"And",matchers);
                        var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunMonthLaborCostDO.SCHEMA_CODE,filter);
                        if (chuanyunFindAllResponse.getReturnData() == null) {
                            //新增
                            chuanyunSaveDTO =chuanYunManager.save(ChuanyunMonthLaborCostDO.SCHEMA_CODE,purcostString,true);
                        }else {
                            List<ChuanyunMonthLaborCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
//                            ChuanyunMonthLaborCostDO chuanyunMonthLaborCostDO = result.get(0);
                            //更新
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunMonthLaborCostDO.SCHEMA_CODE,result.get(0).getObjectId(),purcostString);
                        }
                        if(chuanyunSaveDTO.getSuccessful()){
//                            log.info("新增项目人力成本成功");
                        }else{
                            log.warn("新增项目人力成本失败");
                        }
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            });
        }
    }

    /**
     * 更新项目预决算统计
     */
    private void saveBudgetStatistics() {
        List<ChuanyunBudgetStatisticsDO> list = chuanyunBudgetStatisticsDao.findAll();
        list.forEach(chuanyunBudgetStatisticsDO -> {
            if (!StringUtils.isEmpty(chuanyunBudgetStatisticsDO.getProject())){
                var objectMapper = new ObjectMapper();
                try {
                    //集团总表中查找项目
                    List<String> projectmatchers = new ArrayList<>();
                    projectmatchers.add("Name_2,"+chuanyunBudgetStatisticsDO.getProject());
                    var projectfilter= Filter.instance(0,1,true,"And",projectmatchers);
                    var chuanyunFindResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE,projectfilter);
                    if (chuanyunFindResponse.getReturnData()!=null){
                        List<ChuanyunGroupProjectDO> projectresult=objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                        chuanyunBudgetStatisticsDO.setProject(projectresult.get(0).getObjectId());
                        List<String> matchers = new ArrayList<>();
                        ChuanyunSaveDTO chuanyunSaveDTO;
                        matchers.add("F0000001_2,"+chuanyunBudgetStatisticsDO.getProject());
                        matchers.add("F0000002_2,"+chuanyunBudgetStatisticsDO.getFeeType());
                        var filter= Filter.instance(0,1,true,"And",matchers);
                        var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunBudgetAmountDO.SCHEMA_CODE,filter);
                        BudgetDTO budgetDTO = new BudgetDTO(chuanyunBudgetStatisticsDO);
                        budgetDTO.setDate(LocalDateTime.of(2020,2,1,0,0));
                        var budgetString = objectMapper.writeValueAsString(budgetDTO);
                        if (chuanyunFindAllResponse.getReturnData() != null) {
                            List<ChuanyunBudgetAmountDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                            ChuanyunBudgetAmountDO chuanyunBudgetAmountDO = result.get(0);
                            chuanyunSaveDTO = chuanYunManager.update(ChuanyunBudgetAmountDO.SCHEMA_CODE,chuanyunBudgetAmountDO.getObjectId(),budgetString);
                        }else {
                            chuanyunSaveDTO = chuanYunManager.save(ChuanyunBudgetAmountDO.SCHEMA_CODE,budgetString,true);
                        }
                        if(chuanyunSaveDTO.getSuccessful()){
//                            log.info("新增项目预决算统计成功");
                        }else{
                            log.warn("新增项目预决算统计失败");
                        }
                    }
                } catch (JsonProcessingException | BigSizeException e) {
                    e.printStackTrace();
                }
            }
        });
    }

    @Data
    class PurCostDTO {
        @JsonProperty("F0000001")
        private String projectcode;

        /**
         * 采购成本
         */
        @JsonProperty("F0000002")
        private BigDecimal purcost;

        /**
         * 领料成本
         */
        @JsonProperty("F0000003")
        private BigDecimal issuedcost;

        public PurCostDTO(ChuanyunPurCostDTO chuanyunPurCostDTO) {
            if (null!=chuanyunPurCostDTO) {
                if (null != chuanyunPurCostDTO.getProjectcode()) {
                    this.projectcode = chuanyunPurCostDTO.getProjectcode();
                }
                if (null != chuanyunPurCostDTO.getPurcost()) {
                    this.purcost = chuanyunPurCostDTO.getPurcost();
                }
                if (null != chuanyunPurCostDTO.getIssuedcost()) {
                    this.issuedcost = chuanyunPurCostDTO.getIssuedcost();
                }
            }
        }
    }

    @Data
    class SupplierDTO{
        /**
         * 供应商名称
         */
        @JsonProperty("F0000001")
        private String supplierName;

        /**
         * U9代码
         */
        @JsonProperty("F0000002")
        private String supplierCode;

        public SupplierDTO(ChuanyunSupplierDTO chuanyunSupplierDTO) {
            if (null!=chuanyunSupplierDTO){
                if (null!=chuanyunSupplierDTO.getCode()){
                    this.supplierCode=chuanyunSupplierDTO.getCode();
                }
                if (null!=chuanyunSupplierDTO.getName()){
                    this.supplierName=chuanyunSupplierDTO.getName();
                }
            }
        }
    }

    @Data
    class LaborCostDTO {

        @JsonProperty("F0000005")
        private String projectCode;

        @JsonProperty("F0000003")
        private BigDecimal amount;

        @JsonProperty("F0000002")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        private LocalDateTime logDate;

        @JsonProperty("F0000004")
        private String dataType;

        public LaborCostDTO(ChuanyunLaborCostDTO chuanyunMonthLaborCostDTO) {
            if (null!=chuanyunMonthLaborCostDTO){
                if (null!=chuanyunMonthLaborCostDTO.getCostamount()){
                    this.amount=chuanyunMonthLaborCostDTO.getCostamount();
                }
                if (null!=chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getProjectcode()){
                    this.projectCode=chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getProjectcode();
                }
                if (null!=chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod()){
                    this.logDate=chuanyunMonthLaborCostDTO.getChuanyunLaborCostPrimaryKey().getPeriod();
                }
            }
        }
    }

    @Data
    class BudgetDTO{

        /**
         * 所属项目
         */
        @JsonProperty("F0000001")
        private String project;

        /**
         * 费用类型
         */
        @JsonProperty("F0000002")
        private String feeType;

        /**
         * 预算金额
         */
        @JsonProperty("F0000003")
        private BigDecimal budgetAmount;

        /**
         * 默认日期（4月份）
         */
        @JsonProperty("F0000006")
        @JsonSerialize(using = ChuanyunLocalDateTimeSerializer.class)
        @JsonDeserialize(using = ChuanyunLocalDateTimeDeserializer.class)
        private LocalDateTime date;

        /**
         * 决算金额
         */
        @JsonProperty("F0000004")
        private BigDecimal finalAmount;

        /**
         * 预算绝算差异
         */
        @JsonProperty("F0000005")
        private BigDecimal differenceAmount;

        public BudgetDTO(ChuanyunBudgetStatisticsDO chuanyunBudgetStatisticsDO) {
            if (null!=chuanyunBudgetStatisticsDO){
                if (null!=chuanyunBudgetStatisticsDO.getBudgetAmount()){
                    this.budgetAmount=chuanyunBudgetStatisticsDO.getBudgetAmount();
                }
                if (null!=chuanyunBudgetStatisticsDO.getFinalAmount()){
                    this.finalAmount=chuanyunBudgetStatisticsDO.getFinalAmount();
                }
                if (null!=chuanyunBudgetStatisticsDO.getFeeType()){
                    this.feeType=chuanyunBudgetStatisticsDO.getFeeType();
                }
                if (null!=chuanyunBudgetStatisticsDO.getProject()){
                    this.project=chuanyunBudgetStatisticsDO.getProject();
                }
                if (null!=chuanyunBudgetStatisticsDO.getDifferenceAmount()){
                    this.differenceAmount=chuanyunBudgetStatisticsDO.getDifferenceAmount();
                }
            }
        }
    }

    @Data
    class CashFlowDTO {
        @JsonProperty("F0000001")
        private String projectcode;

        /**
         * 收款金额
         */
        @JsonProperty("F0000002")
        private BigDecimal income;

        /**
         * 付款金额
         */
        @JsonProperty("F0000003")
        private BigDecimal outcome;

        public CashFlowDTO(ChuanyunProjectCashFlowDTO chuanyunProjectCashFlowDTO) {
            if (null!=chuanyunProjectCashFlowDTO) {
                if (null!=chuanyunProjectCashFlowDTO.getProjectcode()) {
                    this.projectcode=chuanyunProjectCashFlowDTO.getProjectcode();
                }
                if (null!=chuanyunProjectCashFlowDTO.getOutcome()) {
                    this.outcome=chuanyunProjectCashFlowDTO.getOutcome();
                }
                if (null!=chuanyunProjectCashFlowDTO.getIncome()) {
                    this.income=chuanyunProjectCashFlowDTO.getIncome();
                }
            }
        }
    }
}
